@@ -23,64 +23,32 @@ The Matplotlib interface can make use of tile objects (defined below) via the
 :meth:`cartopy.mpl.geoaxes.GeoAxes.add_image` method. For example, to add a
 :class:`MapQuest Open Aerial tileset <MapQuestOpenAerial>` to an existing axes
 at zoom level 2, do ``ax.add_image(MapQuestOpenAerial(), 2)``. An example of
-using tiles in this way can be found at :ref:`examples-eyja_volcano`.
+using tiles in this way can be found at the
+:ref:`sphx_glr_gallery_eyja_volcano.py` example.
 
 """
 
 from __future__ import (absolute_import, division, print_function)
 
+from abc import ABCMeta, abstractmethod
+import warnings
+
 from PIL import Image
 import shapely.geometry as sgeom
 import numpy as np
 import six
-import warnings
 
 import cartopy.crs as ccrs
 
 
-class GoogleTiles(object):
+class GoogleWTS(six.with_metaclass(ABCMeta, object)):
     """
     Implement web tile retrieval using the Google WTS coordinate system.
 
     A "tile" in this class refers to the coordinates (x, y, z).
 
     """
-    def __init__(self, desired_tile_form='RGB', style="street",
-                 url=('https://mts0.google.com/vt/lyrs={style}'
-                      '@177000000&hl=en&src=api&x={x}&y={y}&z={z}&s=G')):
-        """
-        Parameters
-        ----------
-        desired_tile_form: optional
-            Defaults to 'RGB'.
-        style: optional
-            The style for the Google Maps tiles.  One of 'street',
-            'satellite', 'terrain', and 'only_streets'.  Defaults to 'street'.
-        url: optional
-            URL pointing to a tile source and containing {x}, {y}, and {z}.
-            Such as: ('https://server.arcgisonline.com/ArcGIS/rest/services/'
-            'World_Shaded_Relief/MapServer/tile/{z}/{y}/{x}.jpg')
-
-        """
-        # Only streets are partly transparent tiles that can be overlayed over
-        # the satellite map to create the known hybrid style from google.
-        styles = ["street", "satellite", "terrain", "only_streets"]
-        style = style.lower()
-        self.url = url
-        if style not in styles:
-            msg = "Invalid style '%s'. Valid styles: %s" % \
-                (style, ", ".join(styles))
-            raise ValueError(msg)
-        self.style = style
-
-        # The 'satellite' and 'terrain' styles require pillow with a jpeg
-        # decoder.
-        if self.style in ["satellite", "terrain"] and \
-                not hasattr(Image.core, "jpeg_decoder") or \
-                not Image.core.jpeg_decoder:
-            msg = "The '%s' style requires pillow with jpeg decoding support."
-            raise ValueError(msg % self.style)
-
+    def __init__(self, desired_tile_form='RGB'):
         self.imgs = []
         self.crs = ccrs.Mercator.GOOGLE
         self.desired_tile_form = desired_tile_form
@@ -183,18 +151,9 @@ class GoogleTiles(object):
 
     _tileextent = tileextent
 
+    @abstractmethod
     def _image_url(self, tile):
-        style_dict = {
-            "street": "m",
-            "satellite": "s",
-            "terrain": "t",
-            "only_streets": "h"}
-        url = self.url.format(
-            style=style_dict[self.style],
-            x=tile[0], X=tile[0],
-            y=tile[1], Y=tile[1],
-            z=tile[2], Z=tile[2])
-        return url
+        pass
 
     def get_image(self, tile):
         if six.PY3:
@@ -214,7 +173,58 @@ class GoogleTiles(object):
         return img, self.tileextent(tile), 'lower'
 
 
-class MapQuestOSM(GoogleTiles):
+class GoogleTiles(GoogleWTS):
+    def __init__(self, desired_tile_form='RGB', style="street",
+                 url=('https://mts0.google.com/vt/lyrs={style}'
+                      '@177000000&hl=en&src=api&x={x}&y={y}&z={z}&s=G')):
+        """
+        Parameters
+        ----------
+        desired_tile_form: optional
+            Defaults to 'RGB'.
+        style: optional
+            The style for the Google Maps tiles.  One of 'street',
+            'satellite', 'terrain', and 'only_streets'.  Defaults to 'street'.
+        url: optional
+            URL pointing to a tile source and containing {x}, {y}, and {z}.
+            Such as: ``'https://server.arcgisonline.com/ArcGIS/rest/services/\
+World_Shaded_Relief/MapServer/tile/{z}/{y}/{x}.jpg'``
+
+        """
+        styles = ["street", "satellite", "terrain", "only_streets"]
+        style = style.lower()
+        self.url = url
+        if style not in styles:
+            msg = "Invalid style '%s'. Valid styles: %s" % \
+                (style, ", ".join(styles))
+            raise ValueError(msg)
+        self.style = style
+
+        # The 'satellite' and 'terrain' styles require pillow with a jpeg
+        # decoder.
+        if self.style in ["satellite", "terrain"] and \
+                not hasattr(Image.core, "jpeg_decoder") or \
+                not Image.core.jpeg_decoder:
+            msg = "The '%s' style requires pillow with jpeg decoding support."
+            raise ValueError(msg % self.style)
+        return super(GoogleTiles, self).__init__(
+            desired_tile_form=desired_tile_form)
+
+    def _image_url(self, tile):
+        style_dict = {
+            "street": "m",
+            "satellite": "s",
+            "terrain": "t",
+            "only_streets": "h"}
+        url = self.url.format(
+            style=style_dict[self.style],
+            x=tile[0], X=tile[0],
+            y=tile[1], Y=tile[1],
+            z=tile[2], Z=tile[2])
+        return url
+
+
+class MapQuestOSM(GoogleWTS):
     # http://developer.mapquest.com/web/products/open/map for terms of use
     # http://devblog.mapquest.com/2016/06/15/
     # modernization-of-mapquest-results-in-changes-to-open-tile-access/
@@ -231,7 +241,7 @@ class MapQuestOSM(GoogleTiles):
         return url
 
 
-class MapQuestOpenAerial(GoogleTiles):
+class MapQuestOpenAerial(GoogleWTS):
     # http://developer.mapquest.com/web/products/open/map for terms of use
     # The following attribution should be included in the resulting image:
     # "Portions Courtesy NASA/JPL-Caltech and U.S. Depart. of Agriculture,
@@ -243,7 +253,7 @@ class MapQuestOpenAerial(GoogleTiles):
         return url
 
 
-class OSM(GoogleTiles):
+class OSM(GoogleWTS):
     # http://developer.mapquest.com/web/products/open/map for terms of use
     def _image_url(self, tile):
         x, y, z = tile
@@ -251,8 +261,40 @@ class OSM(GoogleTiles):
         return url
 
 
-class StamenTerrain(GoogleTiles):
+class Stamen(GoogleWTS):
     """
+    Retrieves tiles from maps.stamen.com. Styles include
+    ``terrain-background``, ``terrain``, ``toner`` and ``watercolor``.
+
+    For a full reference on the styles available please see
+    http://maps.stamen.com. Of particular note are the sub-styles
+    that are made available (e.g. ``terrain`` and ``terrain-background``).
+    To determine the name of the particular [sub-]style you want,
+    follow the link on http://maps.stamen.com to your desired style and
+    observe the style name in the URL. Your style name will be in the
+    form of: ``http://maps.stamen.com/{STYLE_NAME}/#9/37/-122``.
+
+    Except otherwise noted, the Stamen map tile sets are copyright Stamen
+    Design, under a Creative Commons Attribution (CC BY 3.0) license.
+
+    Please see the attribution notice at http://maps.stamen.com on how to
+    attribute this imagery.
+
+    """
+    def __init__(self, style='toner'):
+        super(Stamen, self).__init__()
+        self.style = style
+
+    def _image_url(self, tile):
+        return ('http://tile.stamen.com/{self.style}/{z}/{x}/{y}.png'
+                .format(self=self, x=tile[0], y=tile[1], z=tile[2]))
+
+
+class StamenTerrain(Stamen):
+    """
+    **DEPRECATED:** This class is deprecated. Please use
+    ``Stamen('terrain-background')`` instead.
+
     Terrain tiles defined for the continental United States, and include land
     color and shaded hills. The land colors are a custom palette developed by
     Gem Spear for the National Atlas 1km land cover data set, which defines
@@ -263,21 +305,28 @@ class StamenTerrain(GoogleTiles):
     leave room for foreground data and break up the weirdness of large areas
     of flat, dark green.
 
-    Additional info:
-    http://mike.teczno.com/notes/osm-us-terrain-layer/background.html
-    http://maps.stamen.com/
-    https://wiki.openstreetmap.org/wiki/List_of_OSM_based_Services
-    https://github.com/migurski/DEM-Tools
+    References
+    ----------
+
+     * http://mike.teczno.com/notes/osm-us-terrain-layer/background.html
+     * http://maps.stamen.com/
+     * https://wiki.openstreetmap.org/wiki/List_of_OSM_based_Services
+     * https://github.com/migurski/DEM-Tools
+
 
     """
-    def _image_url(self, tile):
-        x, y, z = tile
-        url = 'http://tile.stamen.com/terrain-background/%s/%s/%s.png' % (
-            z, x, y)
-        return url
+    def __init__(self):
+        warnings.warn(
+            "The StamenTerrain class was deprecated in v0.17. "
+            "Please use Stamen('terrain-background') instead.")
+
+        # NOTE: This subclass of Stamen exists for legacy reasons.
+        # No further Stamen subclasses will be accepted as
+        # they can easily be created in user code with Stamen(style_name).
+        return super(StamenTerrain, self).__init__(style='terrain-background')
 
 
-class MapboxTiles(GoogleTiles):
+class MapboxTiles(GoogleWTS):
     """
     Implement web tile retrieval from Mapbox.
 
@@ -296,8 +345,8 @@ class MapboxTiles(GoogleTiles):
         access_token
             A valid Mapbox API access token.
         map_id
-            A map ID for a publically accessible map. This is the
-            map whose tiles will be retrieved through this process.
+            An ID for a publicly accessible map (provided by Mapbox).
+            This is the map whose tiles will be retrieved through this process.
 
         """
         self.access_token = access_token
@@ -306,14 +355,59 @@ class MapboxTiles(GoogleTiles):
 
     def _image_url(self, tile):
         x, y, z = tile
-        url = ('https://api.tiles.mapbox.com/v4/{mapid}/{z}/{x}/{y}.png?'
-               'access_token={token}'.format(z=z, y=y, x=x,
-                                             mapid=self.map_id,
-                                             token=self.access_token))
+        url = ('https://api.mapbox.com/v4/mapbox.{id}/{z}/{x}/{y}.png'
+               '?access_token={token}'.format(z=z, y=y, x=x,
+                                              id=self.map_id,
+                                              token=self.access_token))
         return url
 
 
-class QuadtreeTiles(GoogleTiles):
+class MapboxStyleTiles(GoogleWTS):
+    """
+    Implement web tile retrieval from a user-defined Mapbox style. For more
+    details on Mapbox styles, see
+    https://www.mapbox.com/studio-manual/overview/map-styling/.
+
+    For terms of service, see https://www.mapbox.com/tos/.
+
+    """
+    def __init__(self, access_token, username, map_id):
+        """
+        Set up a new instance to retrieve tiles from a Mapbox style.
+
+        Access to Mapbox web services requires an access token and a map ID.
+        See https://www.mapbox.com/api-documentation/ for details.
+
+        Parameters
+        ----------
+        access_token
+            A valid Mapbox API access token.
+        username
+            The username for the Mapbox user who defined the Mapbox style.
+        map_id
+            A map ID for a map defined by a Mapbox style. This is the map whose
+            tiles will be retrieved through this process. Note that this style
+            may be private and if your access token does not have permissions
+            to view this style, then map tile retrieval will fail.
+
+        """
+        self.access_token = access_token
+        self.username = username
+        self.map_id = map_id
+        super(MapboxStyleTiles, self).__init__()
+
+    def _image_url(self, tile):
+        x, y, z = tile
+        url = ('https://api.mapbox.com/styles/v1/'
+               '{user}/{mapid}/tiles/256/{z}/{x}/{y}'
+               '?access_token={token}'.format(z=z, y=y, x=x,
+                                              user=self.username,
+                                              mapid=self.map_id,
+                                              token=self.access_token))
+        return url
+
+
+class QuadtreeTiles(GoogleWTS):
     """
     Implement web tile retrieval using the Microsoft WTS quadkey coordinate
     system.
@@ -378,7 +472,7 @@ class QuadtreeTiles(GoogleTiles):
 
     def tileextent(self, quadkey):
         x_y_z = self.quadkey_to_tms(quadkey, google=True)
-        return GoogleTiles.tileextent(self, x_y_z)
+        return GoogleWTS.tileextent(self, x_y_z)
 
     def find_images(self, target_domain, target_z, start_tile=None):
         """
@@ -398,8 +492,8 @@ class QuadtreeTiles(GoogleTiles):
 
         for start_tile in start_tiles:
             start_tile = self.quadkey_to_tms(start_tile, google=True)
-            for tile in GoogleTiles.find_images(self, target_domain, target_z,
-                                                start_tile=start_tile):
+            for tile in GoogleWTS.find_images(self, target_domain, target_z,
+                                              start_tile=start_tile):
                 yield self.tms_to_quadkey(tile, google=True)
 
 
