@@ -45,15 +45,14 @@ from cartopy import config
 import cartopy.crs as ccrs
 import cartopy.feature
 import cartopy.img_transform
-from cartopy.mpl.clip_path import clip_path
 import cartopy.mpl.feature_artist as feature_artist
 import cartopy.mpl.patch as cpatch
 from cartopy.mpl.slippy_image_artist import SlippyImageArtist
 from cartopy.vector_transform import vector_scalar_to_grid
 
 
-assert mpl.__version__ >= '1.3', ('Cartopy is only supported with '
-                                  'Matplotlib 1.3 or greater.')
+assert mpl.__version__ >= '1.5.1', ('Cartopy is only supported with '
+                                    'Matplotlib 1.5.1 or greater.')
 
 
 _PATH_TRANSFORM_CACHE = weakref.WeakKeyDictionary()
@@ -116,6 +115,17 @@ class InterProjectionTransform(mtransforms.Transform):
                                                self.source_projection,
                                                self.target_projection))
 
+    def __eq__(self, other):
+        if not isinstance(other, self.__class__):
+            result = NotImplemented
+        else:
+            result = (self.source_projection == other.source_projection and
+                      self.target_projection == other.target_projection)
+        return result
+
+    def __ne__(self, other):
+        return not self == other
+
     def transform_non_affine(self, xy):
         """
         Transform from source to target coordinates.
@@ -237,18 +247,18 @@ class GeoAxes(matplotlib.axes.Axes):
     when created with the *projection* keyword. For example::
 
         # Set up a standard map for latlon data.
-        geo_axes = pyplot.axes(projection=cartopy.crs.PlateCarree())
+        geo_axes = plt.axes(projection=cartopy.crs.PlateCarree())
 
         # Set up an OSGB map.
-        geo_axes = pyplot.subplot(2, 2, 1, projection=cartopy.crs.OSGB())
+        geo_axes = plt.subplot(2, 2, 1, projection=cartopy.crs.OSGB())
 
     When a source projection is provided to one of it's plotting methods,
     using the *transform* keyword, the standard Matplotlib plot result is
     transformed from source coordinates to the target projection. For example::
 
         # Plot latlon data on an OSGB map.
-        pyplot.axes(projection=cartopy.crs.OSGB())
-        pyplot.contourf(x, y, data, transform=cartopy.crs.PlateCarree())
+        plt.axes(projection=cartopy.crs.OSGB())
+        plt.contourf(x, y, data, transform=cartopy.crs.PlateCarree())
 
     """
     def __init__(self, *args, **kwargs):
@@ -307,14 +317,7 @@ class GeoAxes(matplotlib.axes.Axes):
             # Args and kwargs not allowed.
             assert not bool(args) and not bool(kwargs)
             image = factory
-            try:
-                super(GeoAxes, self).add_image(image)
-            except AttributeError:
-                # If add_image method doesn't exist (only available from
-                # v1.4 onwards) we implement it ourselves.
-                self._set_artist_props(image)
-                self.images.append(image)
-                image._remove_method = lambda h: self.images.remove(h)
+            super(GeoAxes, self).add_image(image)
             return image
 
     @contextlib.contextmanager
@@ -360,8 +363,8 @@ class GeoAxes(matplotlib.axes.Axes):
             self.autoscale_view()
 
         if self.outline_patch.reclip or self.background_patch.reclip:
-            clipped_path = clip_path(self.outline_patch.orig_path,
-                                     self.viewLim)
+            clipped_path = self.outline_patch.orig_path.clip_to_bbox(
+                self.viewLim)
             self.outline_patch._path = clipped_path
             self.background_patch._path = clipped_path
 
@@ -432,14 +435,17 @@ class GeoAxes(matplotlib.axes.Axes):
         ----------
         resolution
             A named resolution to use from the Natural Earth
-            dataset. Currently can be one of "110m", "50m", and "10m".
+            dataset. Currently can be one of "110m", "50m", and "10m",
+            or a Scaler object.
 
         """
         kwargs['edgecolor'] = color
         kwargs['facecolor'] = 'none'
-        feature = cartopy.feature.NaturalEarthFeature('physical', 'coastline',
-                                                      resolution, **kwargs)
-        return self.add_feature(feature)
+        feature = cartopy.feature.COASTLINE
+        if resolution != 'auto':
+            feature = feature.with_scale(resolution)
+
+        return self.add_feature(feature, **kwargs)
 
     def tissot(self, rad_km=500, lons=None, lats=None, n_samples=80, **kwargs):
         """
@@ -562,6 +568,8 @@ class GeoAxes(matplotlib.axes.Axes):
             A collection of shapely geometries.
         crs
             The cartopy CRS in which the provided geometries are defined.
+        styler
+            A callable that returns matplotlib patch styling given a geometry.
 
         Returns
         -------
@@ -576,8 +584,9 @@ class GeoAxes(matplotlib.axes.Axes):
 
 
         """
+        styler = kwargs.pop('styler', None)
         feature = cartopy.feature.ShapelyFeature(geoms, crs, **kwargs)
-        return self.add_feature(feature)
+        return self.add_feature(feature, styler=styler)
 
     def get_extent(self, crs=None):
         """
@@ -625,7 +634,7 @@ class GeoAxes(matplotlib.axes.Axes):
                 raise ValueError('Cannot determine extent in'
                                  ' coordinate system {!r}'.format(crs))
 
-        # Calculate intersection with boundary and project if necesary.
+        # Calculate intersection with boundary and project if necessary.
         boundary_poly = sgeom.Polygon(self.projection.boundary)
         if proj != self.projection:
             # Erode boundary by threshold to avoid transform issues.
@@ -1056,9 +1065,9 @@ class GeoAxes(matplotlib.axes.Axes):
             x_range, y_range = np.diff(target_extent)[::2]
             desired_aspect = x_range / y_range
             if x_range >= y_range:
-                regrid_shape = (target_size * desired_aspect, target_size)
+                regrid_shape = (int(target_size * desired_aspect), target_size)
             else:
-                regrid_shape = (target_size, target_size / desired_aspect)
+                regrid_shape = (target_size, int(target_size / desired_aspect))
         return regrid_shape
 
     def imshow(self, img, *args, **kwargs):
@@ -1391,11 +1400,10 @@ class GeoAxes(matplotlib.axes.Axes):
         result = matplotlib.axes.Axes.contourf(self, *args, **kwargs)
 
         # We need to compute the dataLim correctly for contours.
-        if matplotlib.__version__ >= '1.4':
-            extent = mtransforms.Bbox.union([col.get_datalim(self.transData)
-                                             for col in result.collections
-                                             if col.get_paths()])
-            self.dataLim.update_from_data_xy(extent.get_points())
+        extent = mtransforms.Bbox.union([col.get_datalim(self.transData)
+                                         for col in result.collections
+                                         if col.get_paths()])
+        self.dataLim.update_from_data_xy(extent.get_points())
 
         self.autoscale_view()
         return result
@@ -1417,7 +1425,7 @@ class GeoAxes(matplotlib.axes.Axes):
         if hasattr(t, '_as_mpl_transform'):
             kwargs['transform'] = t._as_mpl_transform(self)
 
-        # exclude Geodetic as a vaild source CS
+        # exclude Geodetic as a valid source CS
         if (isinstance(kwargs.get('transform', None),
                        InterProjectionTransform) and
                 kwargs['transform'].source_projection.is_geodetic()):
@@ -1454,19 +1462,19 @@ class GeoAxes(matplotlib.axes.Axes):
 
     def _pcolormesh_patched(self, *args, **kwargs):
         """
-        A temporary, modified duplicate of
-        :func:`~matplotlib.pyplot.pcolormesh'.
+        A modified duplicate of :func:`~matplotlib.pyplot.pcolormesh'.
 
-        This function contains a workaround for a Matplotlib issue
-        and will be removed once the issue has been resolved.
-        https://github.com/matplotlib/matplotlib/pull/1314
+        This function contains patches for Cartopy-specific behaviour, such as
+        using the transform for limit checks, applying longitude wrapping, etc.
+        See PATCH comments below.
 
         """
         import warnings
         import matplotlib.colors as mcolors
         import matplotlib.collections as mcoll
 
-        if not self._hold:
+        # Remove this check when only MPL >= 3.0 is supported.
+        if not getattr(self, '_hold', True):
             self.cla()
 
         alpha = kwargs.pop('alpha', None)
@@ -1964,14 +1972,8 @@ class GeoAxes(matplotlib.axes.Axes):
 
 # Define the GeoAxesSubplot class, so that a type(ax) will emanate from
 # cartopy.mpl.geoaxes, not matplotlib.axes.
-class GeoAxesSubplot(matplotlib.axes.SubplotBase, GeoAxes):
-    _axes_class = GeoAxes
-
-
-try:
-    matplotlib.axes._subplots._subplot_classes[GeoAxes] = GeoAxesSubplot
-except AttributeError:
-    matplotlib.axes._subplot_classes[GeoAxes] = GeoAxesSubplot
+GeoAxesSubplot = matplotlib.axes.subplot_class_factory(GeoAxes)
+GeoAxesSubplot.__module__ = GeoAxes.__module__
 
 
 def _trigger_patch_reclip(event):
